{% extends "base.html" %}
{% load jmbo_inclusion_tags %} 

{% block content %}
    <div class="page section_news object-list {{ model.lower }}-list">
        
        {% if view_modifier %}
    	    <div class="title">
	    	    {{ title }}
		    </div>      
            {% view_modifier view_modifier %}
        {% endif %}

        {% for object in object_list %}
            {% render_object object "list_item" %}
        {% endfor %}

        {% pager page_obj %}

	</div>
{% endblock %}
